@@ -301,6 +301,66 @@ int LcmFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, c
 	}
 	return 0;
 }
+bool divisors_combine(MathStructure &m, vector<Number> factors, size_t skip, size_t pos = 0, Number nr = nr_one) {
+	for(; pos < factors.size() - skip; pos++) {
+		if(CALCULATOR->aborted()) return false;
+		if(skip > 0) {
+			if(!divisors_combine(m, factors, skip - 1, pos + 1, nr)) return false;
+		}
+		nr *= factors[pos];
+	}
+	bool b = false;
+	for(size_t i = m.size(); i > 0; i--) {
+		if(nr >= m[i - 1].number()) {
+			if(nr != m[i - 1].number()) m.insertChild(nr, i + 1);
+			b = true;
+			break;
+		}
+	}
+	if(!b) m.insertChild(nr, 1);
+	return true;
+}
+DivisorsFunction::DivisorsFunction() : MathFunction("divisors", 1) {
+	setArgumentDefinition(1, new IntegerArgument("", ARGUMENT_MIN_MAX_NONZERO));
+}
+int DivisorsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
+	vector<Number> factors;
+	Number nr(vargs[0].number());
+	nr.abs();
+	mstruct.clearVector();
+	if(nr.isOne()) {
+		mstruct.addChild(nr);
+		return 1;
+	}
+	if(!nr.factorize(factors)) return 0;
+	if(nr.isLessThan(Number(1, 1, factors.size() / 2.5))) {
+		bool b = false;
+		long int n = vargs[0].number().lintValue(&b);
+		if(!b) {
+			if(n < 0) n = -n;
+			mstruct.clearVector();
+			mstruct.addChild(m_one);
+			long int i_last = n / factors[0].lintValue();
+			for(long int i = 2; i <= i_last; i++) {
+				if(CALCULATOR->aborted()) return 0;
+				if(n % i == 0) mstruct.addChild(MathStructure(i, 1L, 0L));
+			}
+			mstruct.addChild(MathStructure(n, 1L, 0L));
+			return 1;
+		}
+	}
+	if(factors.size() == 2) {
+		mstruct.addChild(factors[0]);
+		if(factors[0] != factors[1]) mstruct.addChild(factors[1]);
+	} else if(factors.size() > 2) {
+		for(size_t i = factors.size() - 1; i > 0; i--) {
+			if(!divisors_combine(mstruct, factors, i)) return 0;
+		}
+	}
+	mstruct.insertChild(m_one, 1);
+	mstruct.addChild(nr);
+	return 1;
+}
 
 SignumFunction::SignumFunction() : MathFunction("sgn", 1, 2) {
 	Argument *arg = new NumberArgument("", ARGUMENT_MIN_MAX_NONE, false, false);
@@ -971,7 +1031,7 @@ int BaseFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 						d_i = 0; v++; i++;
 					} else {
 						size_t l = 1;
-						while(i + l < sdigits.length() && sdigits[i + l] <= 0 && (unsigned char) sdigits[i + l] < 0xC0) l++;
+						while(i + l < sdigits.length() && (signed char) sdigits[i + l] <= 0 && (unsigned char) sdigits[i + l] < 0xC0) l++;
 						if(d_i == vdigits.size()) vdigits.resize(d_i + 1);
 						vdigits[d_i][sdigits.substr(i, l)] = v;
 						i += l;
@@ -982,7 +1042,7 @@ int BaseFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 				i_dot = number.length();
 				for(size_t i = 0; i < number.length();) {
 					size_t l = 1;
-					while(i + l < number.length() && number[i + l] <= 0 && (unsigned char) number[i + l] < 0xC0) l++;
+					while(i + l < number.length() && (signed char) number[i + l] <= 0 && (unsigned char) number[i + l] < 0xC0) l++;
 					for(d_i = 0; d_i < vdigits.size(); d_i++) {
 						unordered_map<string, long int>::iterator it = vdigits[d_i].find(number.substr(i, l));
 						if(it != vdigits[d_i].end()) {
@@ -1002,35 +1062,35 @@ int BaseFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 			} else if(idigits == 4 || idigits == 5) {
 				for(size_t i = 0; i < number.length();) {
 					size_t l = 1;
-					while(i + l < number.length() && number[i + l] <= 0 && (unsigned char) number[i + l] < 0xC0) l++;
+					while(i + l < number.length() && (signed char) number[i + l] <= 0 && (unsigned char) number[i + l] < 0xC0) l++;
 					char c = 0;
 					if(l == 1) {
 						c = number[i];
-					} else if(l == 2 && number[i] == -61) {
-						if(number[i + 1] >= -128 && number[i + 1] <= -122) c = 'A';
-						else if(number[i + 1] == -121) c = 'C';
-						else if(number[i + 1] >= -120 && number[i + 1] <= -117) c = 'E';
-						else if(number[i + 1] >= -116 && number[i + 1] <= -113) c = 'I';
-						//else if(number[i + 1] == -112) c = 'D';
-						else if(number[i + 1] == -111) c = 'N';
-						else if(number[i + 1] >= -110 && number[i + 1] <= -106) c = 'O';
-						else if(number[i + 1] == -104) c = 'O';
-						else if(number[i + 1] >= -103 && number[i + 1] <= -100) c = 'U';
-						else if(number[i + 1] == -99) c = 'Y';
-						//else if(number[i + 1] == -98) c = 'T';
-						else if(number[i + 1] == -97) c = 'S';
-						else if(number[i + 1] >= -96 && number[i + 1] <= -90) c = 'a';
-						else if(number[i + 1] == -89) c = 'c';
-						else if(number[i + 1] >= -88 && number[i + 1] <= -85) c = 'e';
-						else if(number[i + 1] >= -84 && number[i + 1] <= -81) c = 'i';
-						//else if(number[i + 1] == -80) c = 'd';
-						else if(number[i + 1] == -79) c = 'n';
-						else if(number[i + 1] >= -78 && number[i + 1] <= -74) c = 'o';
-						else if(number[i + 1] == -72) c = 'o';
-						else if(number[i + 1] >= -71 && number[i + 1] <= -68) c = 'u';
-						else if(number[i + 1] == -67) c = 'y';
-						//else if(number[i + 1] == -66) c = 't';
-						else if(number[i + 1] == -65) c = 'y';
+					} else if(l == 2 && (signed char) number[i] == -61) {
+						if((signed char) number[i + 1] >= -128 && (signed char) number[i + 1] <= -122) c = 'A';
+						else if((signed char) number[i + 1] == -121) c = 'C';
+						else if((signed char) number[i + 1] >= -120 && (signed char) number[i + 1] <= -117) c = 'E';
+						else if((signed char) number[i + 1] >= -116 && (signed char) number[i + 1] <= -113) c = 'I';
+						//else if((signed char) number[i + 1] == -112) c = 'D';
+						else if((signed char) number[i + 1] == -111) c = 'N';
+						else if((signed char) number[i + 1] >= -110 && (signed char) number[i + 1] <= -106) c = 'O';
+						else if((signed char) number[i + 1] == -104) c = 'O';
+						else if((signed char) number[i + 1] >= -103 && (signed char) number[i + 1] <= -100) c = 'U';
+						else if((signed char) number[i + 1] == -99) c = 'Y';
+						//else if((signed char) number[i + 1] == -98) c = 'T';
+						else if((signed char) number[i + 1] == -97) c = 'S';
+						else if((signed char) number[i + 1] >= -96 && (signed char) number[i + 1] <= -90) c = 'a';
+						else if((signed char) number[i + 1] == -89) c = 'c';
+						else if((signed char) number[i + 1] >= -88 && (signed char) number[i + 1] <= -85) c = 'e';
+						else if((signed char) number[i + 1] >= -84 && (signed char) number[i + 1] <= -81) c = 'i';
+						//else if((signed char) number[i + 1] == -80) c = 'd';
+						else if((signed char) number[i + 1] == -79) c = 'n';
+						else if((signed char) number[i + 1] >= -78 && (signed char) number[i + 1] <= -74) c = 'o';
+						else if((signed char) number[i + 1] == -72) c = 'o';
+						else if((signed char) number[i + 1] >= -71 && (signed char) number[i + 1] <= -68) c = 'u';
+						else if((signed char) number[i + 1] == -67) c = 'y';
+						//else if((signed char) number[i + 1] == -66) c = 't';
+						else if((signed char) number[i + 1] == -65) c = 'y';
 					}
 					if(c != 0) {
 						if(c == '0') digits.push_back(0);
@@ -1065,7 +1125,7 @@ int BaseFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 						b_minus = !b_minus;
 					} else {
 						string str_char = number.substr(i, 1);
-						while(i + 1 < number.length() && number[i + 1] < 0 && number[i + 1] && (unsigned char) number[i + 1] < 0xC0) {
+						while(i + 1 < number.length() && (signed char) number[i + 1] < 0 && number[i + 1] && (unsigned char) number[i + 1] < 0xC0) {
 							i++;
 							str_char += number[i];
 						}
@@ -1098,7 +1158,7 @@ int BaseFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 							i = i2 - 1;
 							b_esc = true;
 						}
-						if(digits.empty() && number[i] == (char) -30 && i + 3 < number.length() && number[i + 1] == (char) -120 && number[i + 2] == (char) -110) {
+						if(digits.empty() && (signed char) number[i] == (char) -30 && i + 3 < number.length() && number[i + 1] == (char) -120 && number[i + 2] == (char) -110) {
 							i += 2;
 							b_minus = !b_minus;
 							b_esc = true;
@@ -1768,7 +1828,18 @@ int IsNumberFunction::calculate(MathStructure &mstruct, const MathStructure &var
 	return 1;
 }
 
-#define IS_NUMBER_FUNCTION(x, y) x::x() : MathFunction(#y, 1) {} int x::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {mstruct = vargs[0]; if(!mstruct.isNumber()) mstruct.eval(eo); if(mstruct.isNumber() && mstruct.number().y()) {mstruct.number().setTrue();} else {mstruct.clear(); mstruct.number().setFalse();} return 1;}
+#define IS_NUMBER_FUNCTION(x, y) x::x() : MathFunction(#y, 1) {\
+	Argument *arg = new Argument();\
+	arg->setHandleVector(true);\
+	setArgumentDefinition(1, arg);\
+}\
+int x::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {\
+	mstruct = vargs[0];\
+	if(!mstruct.isNumber()) mstruct.eval(eo);\
+	if(mstruct.isVector()) return -1;\
+	if(mstruct.isNumber() && mstruct.number().y()) {mstruct.number().setTrue();} else {mstruct.clear(); mstruct.number().setFalse();}\
+	return 1;\
+}
 
 IS_NUMBER_FUNCTION(IsIntegerFunction, isInteger)
 IS_NUMBER_FUNCTION(IsRealFunction, isReal)
