@@ -31,8 +31,8 @@
 #include <stdint.h>
 
 #define QALCULATE_MAJOR_VERSION (4)
-#define QALCULATE_MINOR_VERSION (0)
-#define QALCULATE_MICRO_VERSION (0)
+#define QALCULATE_MINOR_VERSION (1)
+#define QALCULATE_MICRO_VERSION (1)
 
 static std::string empty_string;
 
@@ -788,6 +788,8 @@ extern Calculator *calculator;
 #define SIGN_PLUSMINUS			"±"
 
 #define THIN_SPACE " "
+#define NNBSP " "
+#define NBSP " "
 
 #define ID_WRAP_LEFT_CH		'{'
 #define ID_WRAP_RIGHT_CH	'}'
